.g-job-info-key
  font-weight bold
  margin 12px 8px 0 0
  font-weight 16px

  &.inline
    display inline-block

.g-job-info-value.inline
  display inline-block

.g-job-status-badge
  border-radius 2px
  font-weight bold
  display inline
  padding 4px 9px 4px 4px
  text-transform uppercase
  font-size 13px

.g-job-color-inactive
  background-color #ccc
  color #555

.g-job-color-queued
  background-color #dbc345
  color white

.g-job-color-running
  background-color #6666d5
  color white

.g-job-color-success
  background-color #53b653
  color white

.g-job-color-error
  background-color #d44
  color white

.g-job-color-canceled
  background-color #545
  color white

.g-job-timeline-container
  margin-top 5px
