ul.g-admin-options
  list-style-type none
  font-size 18px
  padding-left 0
  maring 0

  >li
    display block
    padding 12px 10px
    font-weight bold
    color #666

  >li:hover
    background-color #f4f4f4
    cursor pointer
    color #222
