.g-item-geospatial
  margin-top 20px

.g-item-geospatial-map
  visibility hidden

.g-item-geospatial-header
  background-color #f0f0f0
  color #555
  font-size 16px
  font-weight bold
  padding 5px 6px

.g-item-geospatial-element
  border-bottom 1px solid #f0f0f0
  color #777
  font-size 14px
  height 25px
  margin 5px 3px 3px

.g-item-geospatial-key, .g-item-geospatial-type, .g-item-geospatial-coordinates
  padding 5px

.g-item-geospatial-key
  font-weight bold
  width 150px
