.g-widget-metadata-header
  background-color #f0f0f0
  padding 5px 6px
  color #555
  font-size 16px
  font-weight bold

.g-widget-metadata-row
  margin 3px
  margin-top 5px
  border-bottom 1px solid #f0f0f0
  min-height 25px
  font-size 14px
  color #777

.g-widget-metadata-key, .g-widget-metadata-value
  margin 5px
  display inline-block
  overflow hidden
  text-overflow ellipsis

.g-widget-metadata-key
  font-weight bold
  width 30%

.g-widget-metadata-key span.small
  float right

.g-widget-metadata-value
  width 60%

.g-widget-metadata-value pre
  padding 0px
  font-size 14px
  color #777
  background-color transparent
  border none
  max-height 320px
  overflow auto
  margin-bottom 0

.g-json-editor
  width 54%
  height 400px
  display inline-block
  background-color white
  vertical-align middle
  margin-left 5px
  margin-right 5px

.jsoneditor
  border 2px solid #e3e3e3

.jsoneditor .menu
  background-color #fbfbfb
  border-bottom 1px solid #f0f0f0

.jsoneditor .menu button
  background #fbfbfb url("~jsoneditor/dist/img/jsoneditor-icons.png")
  border 1px solid #f0f0f0

.jsoneditor .menu button:disabled
  background #fbfbfb url("~jsoneditor/dist/img/jsoneditor-icons.png")
  border 1px solid #f0f0f0

.jsoneditor .search .frame
  border 1px solid #f0f0f0

.g-json-editor .ace_editor, .g-json-editor textarea
  height 400px !important

.g-json-editor .ace_content *
  font-family monospace !important

.jsoneditor .menu button:hover, .jsoneditor .menu button:focus
  background-color #f0f0f0

.jsoneditor-contextmenu ul li .selected
  background-color #eaebea

.jsoneditor .menu .search button
  border none
  background-color transparent

button.g-widget-metadata-add-button
  float right
  margin-right 10px
  padding 2px 3px
  margin-top 4px

.g-widget-metadata-container .g-widget-metadata-edit-button
  margin 2px 8px 0 0
  float right
  font-size 12px
  padding 3px

.g-widget-metadata-row.editing
  .g-widget-metadata-key-input, .g-widget-metadata-value-input
    height 24px
    margin-left 5px
    margin-right 5px
    display inline-block
    vertical-align middle

  .g-widget-metadata-key-input
    width 30%

  .g-widget-metadata-value-input
    width 54%

  .g-widget-metadata-cancel-button, .g-widget-metadata-save-button, .g-widget-metadata-delete-button, .g-widget-metadata-toggle-button
    margin-right 4px
    margin-top 1px
    display inline-block
    height 24px
    font-size 12px
    padding 2px
